@@ -41,18 +41,24 @@
 
 #ifndef QT_NO_IMAGEFORMAT_XPM
 
-#include <private/qcolor_p.h>
 #include <qbytearraymatcher.h>
+#include <qdebug.h>
 #include <qimage.h>
+#include <qloggingcategory.h>
 #include <qmap.h>
-#include <qregexp.h>
 #include <qtextstream.h>
 #include <qvariant.h>
 
+#include <private/qcolor_p.h>
+#include <private/qduplicatetracker_p.h> // for easier std::pmr detection
+
 #include <algorithm>
+#include <array>
 
 QT_BEGIN_NAMESPACE
 
+Q_DECLARE_LOGGING_CATEGORY(lcImageIo)
+
 static quint64 xpmHash(const QString &str)
 {
     unsigned int hashValue = 0;
@@ -766,17 +772,26 @@ static QString fbname(const QString &fileName) // get file basename (sort of)
 {
     QString s = fileName;
     if (!s.isEmpty()) {
-        int i;
-        if ((i = s.lastIndexOf(QLatin1Char('/'))) >= 0)
-            s = s.mid(i);
-        if ((i = s.lastIndexOf(QLatin1Char('\\'))) >= 0)
-            s = s.mid(i);
-        QRegExp r(QLatin1String("[a-zA-Z][a-zA-Z0-9_]*"));
-        int p = r.indexIn(s);
-        if (p == -1)
+        int i = qMax(s.lastIndexOf(QLatin1Char('/')), s.lastIndexOf(QLatin1Char('\\')));
+        if (i < 0)
+            i = 0;
+        auto isAsciiLetterOrNumber = [](QChar ch) -> bool {
+            return (ch.unicode() >= '0' && ch.unicode() <= '9') ||
+                    (ch.unicode() >= 'A' && ch.unicode() <= 'Z') ||
+                    (ch.unicode() >= 'a' && ch.unicode() <= 'z') ||
+                    ch.unicode() == '_';
+        };
+        int start = -1;
+        for (; i < s.length(); ++i) {
+            if (isAsciiLetterOrNumber(s.at(i))) {
+                start = i;
+            } else if (start > 0)
+                break;
+        }
+        if (start < 0)
             s.clear();
         else
-            s = s.mid(p, r.matchedLength());
+            s = s.mid(start, i - start);
     }
     if (s.isEmpty())
         s = QString::fromLatin1("dummy");
@@ -873,11 +888,8 @@ static bool read_xpm_body(
     // create it in correct format (Format_RGB32 vs Format_ARGB32,
     // depending on absence or presence of "c none", respectively)
     if (ncols <= 256) {
-        if (image.size() != QSize(w, h) || image.format() != QImage::Format_Indexed8) {
-            image = QImage(w, h, QImage::Format_Indexed8);
-            if (image.isNull())
-                return false;
-        }
+        if (!QImageIOHandler::allocateImage(QSize(w, h), QImage::Format_Indexed8, &image))
+            return false;
         image.setColorCount(ncols);
     }
 
@@ -887,7 +899,7 @@ static bool read_xpm_body(
 
     for(currentColor=0; currentColor < ncols; ++currentColor) {
         if (!read_xpm_string(buf, device, source, index, state)) {
-            qWarning("QImage: XPM color specification missing");
+            qCWarning(lcImageIo, "XPM color specification missing");
             return false;
         }
         QByteArray index;
@@ -902,7 +914,7 @@ static bool read_xpm_body(
         if (i < 0)
             i = tokens.indexOf("m");
         if (i < 0) {
-            qWarning("QImage: XPM color specification is missing: %s", buf.constData());
+            qCWarning(lcImageIo, "XPM color specification is missing: %s", buf.constData());
             return false;        // no c/g/g4/m specification at all
         }
         QByteArray color;
@@ -910,7 +922,7 @@ static bool read_xpm_body(
             color.append(tokens.at(i));
         }
         if (color.isEmpty()) {
-            qWarning("QImage: XPM color value is missing from specification: %s", buf.constData());
+            qCWarning(lcImageIo, "XPM color value is missing from specification: %s", buf.constData());
             return false;        // no color value
         }
         buf = color;
@@ -924,7 +936,7 @@ static bool read_xpm_body(
                 colorMap.insert(xpmHash(QLatin1String(index.constData())), 0);
             }
         } else {
-            QRgb c_rgb;
+            QRgb c_rgb = 0;
             if (((buf.length()-1) % 3) && (buf[0] == '#')) {
                 buf.truncate(((buf.length()-1) / 4 * 3) + 1); // remove alpha channel left by imagemagick
             }
@@ -946,17 +958,14 @@ static bool read_xpm_body(
         // Now we can create 32-bit image of appropriate format
         QImage::Format format = hasTransparency ?
                                 QImage::Format_ARGB32 : QImage::Format_RGB32;
-        if (image.size() != QSize(w, h) || image.format() != format) {
-            image = QImage(w, h, format);
-            if (image.isNull())
-                return false;
-        }
+        if (!QImageIOHandler::allocateImage(QSize(w, h), format, &image))
+            return false;
     }
 
     // Read pixels
     for(int y=0; y<h; y++) {
         if (!read_xpm_string(buf, device, source, index, state)) {
-            qWarning("QImage: XPM pixels missing on image line %d", y);
+            qCWarning(lcImageIo, "XPM pixels missing on image line %d", y);
             return false;
         }
         if (image.depth() == 8) {
@@ -974,7 +983,7 @@ static bool read_xpm_body(
             } else {
                 char b[16];
                 b[cpp] = '\0';
-                for (x=0; x<w && d<end; x++) {
+                for (x = 0; x < w && d + cpp <= end; x++) {
                     memcpy(b, (char *)d, cpp);
                     *p++ = (uchar)colorMap[xpmHash(b)];
                     d += cpp;
@@ -982,7 +991,7 @@ static bool read_xpm_body(
             }
             // avoid uninitialized memory for malformed xpms
             if (x < w) {
-                qWarning("QImage: XPM pixels missing on image line %d (possibly a C++ trigraph).", y);
+                qCWarning(lcImageIo, "XPM pixels missing on image line %d (possibly a C++ trigraph).", y);
                 memset(p, 0, w - x);
             }
         } else {
@@ -992,14 +1001,14 @@ static bool read_xpm_body(
             int x;
             char b[16];
             b[cpp] = '\0';
-            for (x=0; x<w && d<end; x++) {
+            for (x = 0; x < w && d + cpp <= end; x++) {
                 memcpy(b, (char *)d, cpp);
                 *p++ = (QRgb)colorMap[xpmHash(b)];
                 d += cpp;
             }
             // avoid uninitialized memory for malformed xpms
             if (x < w) {
-                qWarning("QImage: XPM pixels missing on image line %d (possibly a C++ trigraph).", y);
+                qCWarning(lcImageIo, "XPM pixels missing on image line %d (possibly a C++ trigraph).", y);
                 memset(p, 0, (w - x)*4);
             }
         }
@@ -1039,7 +1048,7 @@ bool qt_read_xpm_image_or_array(QIODevice *device, const char * const * source,
         if ((readBytes = device->readLine(buf.data(), buf.size())) < 0)
             return false;
 
-        static Q_RELAXED_CONSTEXPR auto matcher = qMakeStaticByteArrayMatcher("/* XPM");
+        static constexpr auto matcher = qMakeStaticByteArrayMatcher("/* XPM");
 
         if (matcher.indexIn(buf) != 0) {
             while (readBytes > 0) {
@@ -1056,15 +1065,23 @@ bool qt_read_xpm_image_or_array(QIODevice *device, const char * const * source,
     return read_xpm_body(device, source, index, state, cpp, ncols, w, h, image);
 }
 
-static const char* xpm_color_name(int cpp, int index)
+namespace {
+template <size_t N>
+struct CharBuffer : std::array<char, N>
+{
+    CharBuffer() {} // avoid value-initializing the whole array
+};
+}
+
+static const char* xpm_color_name(int cpp, int index, CharBuffer<5> && returnable = {})
 {
-    static char returnable[5];
     static const char code[] = ".#abcdefghijklmnopqrstuvwxyzABCD"
                                "EFGHIJKLMNOPQRSTUVWXYZ0123456789";
     // cpp is limited to 4 and index is limited to 64^cpp
     if (cpp > 1) {
         if (cpp > 2) {
             if (cpp > 3) {
+                returnable[4] = '\0';
                 returnable[3] = code[index % 64];
                 index /= 64;
             } else
@@ -1084,7 +1101,7 @@ static const char* xpm_color_name(int cpp, int index)
         returnable[1] = '\0';
     returnable[0] = code[index];
 
-    return returnable;
+    return returnable.data();
 }
 
 
@@ -1100,18 +1117,25 @@ static bool write_xpm_image(const QImage &sourceImage, QIODevice *device, const
     else
         image = sourceImage;
 
-    QMap<QRgb, int> colorMap;
+#ifdef __cpp_lib_memory_resource
+    char buffer[1024];
+    std::pmr::monotonic_buffer_resource res{&buffer, sizeof buffer};
+    std::pmr::map<QRgb, int> colorMap(&res);
+#else
+    std::map<QRgb, int> colorMap;
+#endif
 
-    int w = image.width(), h = image.height(), ncolors = 0;
-    int x, y;
+    const int w = image.width();
+    const int h = image.height();
+    int ncolors = 0;
 
     // build color table
-    for(y=0; y<h; y++) {
+    for (int y = 0; y < h; ++y) {
         const QRgb *yp = reinterpret_cast<const QRgb *>(image.constScanLine(y));
-        for(x=0; x<w; x++) {
-            QRgb color = *(yp + x);
-            if (!colorMap.contains(color))
-                colorMap.insert(color, ncolors++);
+        for (int x = 0; x < w; ++x) {
+            const auto [it, inserted] = colorMap.try_emplace(yp[x], ncolors);
+            if (inserted)
+                ++ncolors;
         }
     }
 
@@ -1121,8 +1145,11 @@ static bool write_xpm_image(const QImage &sourceImage, QIODevice *device, const
         ++cpp;
         // limit to 4 characters per pixel
         // 64^4 colors is enough for a 4096x4096 image
-         if (cpp > 4)
-            break;
+         if (cpp > 4) {
+             qCWarning(lcImageIo, "Qt does not support writing XPM images with more than "
+                       "64^4 colors (requested: %d colors).", ncolors);
+             return false;
+         }
     }
 
     // write header
@@ -1132,36 +1159,21 @@ static bool write_xpm_image(const QImage &sourceImage, QIODevice *device, const
       << '\"' << w << ' ' << h << ' ' << ncolors << ' ' << cpp << '\"';
 
     // write palette
-    QMap<QRgb, int>::Iterator c = colorMap.begin();
-    while (c != colorMap.end()) {
-        QRgb color = c.key();
+    for (const auto &[color, index] : colorMap) {
         const QString line = image.format() != QImage::Format_RGB32 && !qAlpha(color)
-            ? QString::asprintf("\"%s c None\"", xpm_color_name(cpp, *c))
-            : QString::asprintf("\"%s c #%02x%02x%02x\"", xpm_color_name(cpp, *c),
+            ? QString::asprintf("\"%s c None\"", xpm_color_name(cpp, index))
+            : QString::asprintf("\"%s c #%02x%02x%02x\"", xpm_color_name(cpp, index),
                                 qRed(color), qGreen(color), qBlue(color));
-        ++c;
         s << ',' << Qt::endl << line;
     }
 
     // write pixels, limit to 4 characters per pixel
-    QByteArray line;
-    for(y=0; y<h; y++) {
-        line.clear();
+    for (int y = 0; y < h; ++y) {
+        s << ',' << Qt::endl << '\"';
         const QRgb *yp = reinterpret_cast<const QRgb *>(image.constScanLine(y));
-        for(x=0; x<w; x++) {
-            int color = (int)(*(yp + x));
-            const QByteArray chars(xpm_color_name(cpp, colorMap[color]));
-            line.append(chars[0]);
-            if (cpp > 1) {
-                line.append(chars[1]);
-                if (cpp > 2) {
-                    line.append(chars[2]);
-                    if (cpp > 3)
-                        line.append(chars[3]);
-                }
-            }
-        }
-        s << ',' << Qt::endl << '\"' << line << '\"';
+        for (int x = 0; x < w; ++x)
+            s << xpm_color_name(cpp, colorMap[yp[x]]);
+        s << '\"';
     }
     s << "};" << Qt::endl;
     return (s.status() == QTextStream::Ok);
@@ -1175,7 +1187,7 @@ QXpmHandler::QXpmHandler()
 bool QXpmHandler::readHeader()
 {
     state = Error;
-    if (!read_xpm_header(device(), 0, index, buffer, &cpp, &ncols, &width, &height))
+    if (!read_xpm_header(device(), nullptr, index, buffer, &cpp, &ncols, &width, &height))
         return false;
     state = ReadHeader;
     return true;
@@ -1191,7 +1203,7 @@ bool QXpmHandler::readImage(QImage *image)
         return false;
     }
 
-    if (!read_xpm_body(device(), 0, index, buffer, cpp, ncols, width, height, *image)) {
+    if (!read_xpm_body(device(), nullptr, index, buffer, cpp, ncols, width, height, *image)) {
         state = Error;
         return false;
     }
@@ -1216,7 +1228,7 @@ bool QXpmHandler::canRead() const
 bool QXpmHandler::canRead(QIODevice *device)
 {
     if (!device) {
-        qWarning("QXpmHandler::canRead() called with no device");
+        qCWarning(lcImageIo, "QXpmHandler::canRead() called with no device");
         return false;
     }
 
@@ -1280,13 +1292,6 @@ void QXpmHandler::setOption(ImageOption option, const QVariant &value)
         fileName = value.toString();
 }
 
-#if QT_DEPRECATED_SINCE(5, 13)
-QByteArray QXpmHandler::name() const
-{
-    return "xpm";
-}
-#endif
-
 QT_END_NAMESPACE
 
 #endif // QT_NO_IMAGEFORMAT_XPM
