@@ -147,7 +147,7 @@ QT_BEGIN_NAMESPACE
 */
 
 QAbstractSpinBox::QAbstractSpinBox(QWidget *parent)
-    : QWidget(*new QAbstractSpinBoxPrivate, parent, 0)
+    : QWidget(*new QAbstractSpinBoxPrivate, parent, { })
 {
     Q_D(QAbstractSpinBox);
     d->init();
@@ -157,7 +157,7 @@ QAbstractSpinBox::QAbstractSpinBox(QWidget *parent)
     \internal
 */
 QAbstractSpinBox::QAbstractSpinBox(QAbstractSpinBoxPrivate &dd, QWidget *parent)
-    : QWidget(dd, parent, 0)
+    : QWidget(dd, parent, { })
 {
     Q_D(QAbstractSpinBox);
     d->init();
@@ -420,7 +420,7 @@ bool QAbstractSpinBox::isAccelerated() const
      \since 5.3
 
 
-     This property holds whether a thousands separator is enabled. By default this
+     \brief whether a thousands separator is enabled. By default this
      property is false.
 */
 bool QAbstractSpinBox::isGroupSeparatorShown() const
@@ -570,7 +570,7 @@ void QAbstractSpinBox::clear()
 QAbstractSpinBox::StepEnabled QAbstractSpinBox::stepEnabled() const
 {
     Q_D(const QAbstractSpinBox);
-    if (d->readOnly || d->type == QVariant::Invalid)
+    if (d->readOnly || d->type == QMetaType::UnknownType)
         return StepNone;
     if (d->wrapping)
         return StepEnabled(StepUpEnabled | StepDownEnabled);
@@ -671,7 +671,8 @@ void QAbstractSpinBox::stepBy(int steps)
     } else if (e == AlwaysEmit) {
         d->emitSignals(e, old);
     }
-    selectAll();
+    if (style()->styleHint(QStyle::SH_SpinBox_SelectOnStep, nullptr, this, nullptr))
+        selectAll();
 }
 
 /*!
@@ -723,7 +724,7 @@ void QAbstractSpinBox::setLineEdit(QLineEdit *lineEdit)
     d->edit->setFocusProxy(this);
     d->edit->setAcceptDrops(false);
 
-    if (d->type != QVariant::Invalid) {
+    if (d->type != QMetaType::UnknownType) {
         connect(d->edit, SIGNAL(textChanged(QString)),
                 this, SLOT(_q_editorTextChanged(QString)));
         connect(d->edit, SIGNAL(cursorPositionChanged(int,int)),
@@ -795,7 +796,7 @@ bool QAbstractSpinBox::event(QEvent *event)
     case QEvent::HoverEnter:
     case QEvent::HoverLeave:
     case QEvent::HoverMove:
-        d->updateHoverControl(static_cast<const QHoverEvent *>(event)->pos());
+        d->updateHoverControl(static_cast<const QHoverEvent *>(event)->position().toPoint());
         break;
     case QEvent::ShortcutOverride:
         if (d->edit->event(event))
@@ -848,9 +849,9 @@ void QAbstractSpinBox::changeEvent(QEvent *event)
 
     switch (event->type()) {
         case QEvent::StyleChange:
-            d->spinClickTimerInterval = style()->styleHint(QStyle::SH_SpinBox_ClickAutoRepeatRate, 0, this);
+            d->spinClickTimerInterval = style()->styleHint(QStyle::SH_SpinBox_ClickAutoRepeatRate, nullptr, this);
             d->spinClickThresholdTimerInterval =
-                style()->styleHint(QStyle::SH_SpinBox_ClickAutoRepeatThreshold, 0, this);
+                style()->styleHint(QStyle::SH_SpinBox_ClickAutoRepeatThreshold, nullptr, this);
             if (d->edit)
                 d->edit->setFrame(!style()->styleHint(QStyle::SH_SpinBox_ButtonsInsideFrame, nullptr, this));
             d->stepModifier = static_cast<Qt::KeyboardModifier>(style()->styleHint(QStyle::SH_SpinBox_StepModifier, nullptr, this));
@@ -924,8 +925,7 @@ QSize QAbstractSpinBox::sizeHint() const
         QStyleOptionSpinBox opt;
         initStyleOption(&opt);
         QSize hint(w, h);
-        d->cachedSizeHint = style()->sizeFromContents(QStyle::CT_SpinBox, &opt, hint, this)
-                            .expandedTo(QApplication::globalStrut());
+        d->cachedSizeHint = style()->sizeFromContents(QStyle::CT_SpinBox, &opt, hint, this);
     }
     return d->cachedSizeHint;
 }
@@ -966,8 +966,7 @@ QSize QAbstractSpinBox::minimumSizeHint() const
         initStyleOption(&opt);
         QSize hint(w, h);
 
-        d->cachedMinimumSizeHint = style()->sizeFromContents(QStyle::CT_SpinBox, &opt, hint, this)
-                                   .expandedTo(QApplication::globalStrut());
+        d->cachedMinimumSizeHint = style()->sizeFromContents(QStyle::CT_SpinBox, &opt, hint, this);
     }
     return d->cachedMinimumSizeHint;
 }
@@ -1012,6 +1011,8 @@ void QAbstractSpinBox::keyPressEvent(QKeyEvent *event)
 {
     Q_D(QAbstractSpinBox);
 
+    d->keyboardModifiers = event->modifiers();
+
     if (!event->text().isEmpty() && d->edit->cursorPosition() < d->prefix.size())
         d->edit->setCursorPosition(d->prefix.size());
 
@@ -1043,13 +1044,13 @@ void QAbstractSpinBox::keyPressEvent(QKeyEvent *event)
             steps *= 10;
         if (!up)
             steps *= -1;
-        if (style()->styleHint(QStyle::SH_SpinBox_AnimateButton, 0, this)) {
+        if (style()->styleHint(QStyle::SH_SpinBox_AnimateButton, nullptr, this)) {
             d->buttonState = (Keyboard | (up ? Up : Down));
         }
         if (d->spinClickTimerId == -1)
             stepBy(steps);
-        if(event->isAutoRepeat() && !isPgUpOrDown) {
-            if(d->spinClickThresholdTimerId == -1 && d->spinClickTimerId == -1) {
+        if (event->isAutoRepeat() && !isPgUpOrDown) {
+            if (d->spinClickThresholdTimerId == -1 && d->spinClickTimerId == -1) {
                 d->updateState(up, true);
             }
         }
@@ -1152,6 +1153,7 @@ void QAbstractSpinBox::keyReleaseEvent(QKeyEvent *event)
 {
     Q_D(QAbstractSpinBox);
 
+    d->keyboardModifiers = event->modifiers();
     if (d->buttonState & Keyboard && !event->isAutoRepeat())  {
         d->reset();
     } else {
@@ -1284,7 +1286,7 @@ void QAbstractSpinBox::timerEvent(QTimerEvent *event)
     }
 
     if (doStep) {
-        const bool increaseStepRate = QGuiApplication::keyboardModifiers() & d->stepModifier;
+        const bool increaseStepRate = d->keyboardModifiers & d->stepModifier;
         const StepEnabled st = stepEnabled();
         if (d->buttonState & Up) {
             if (!(st & StepUpEnabled)) {
@@ -1361,7 +1363,8 @@ void QAbstractSpinBox::mouseMoveEvent(QMouseEvent *event)
 {
     Q_D(QAbstractSpinBox);
 
-    d->updateHoverControl(event->pos());
+    d->keyboardModifiers = event->modifiers();
+    d->updateHoverControl(event->position().toPoint());
 
     // If we have a timer ID, update the state
     if (d->spinClickTimerId != -1 && d->buttonSymbols != NoButtons) {
@@ -1384,11 +1387,12 @@ void QAbstractSpinBox::mousePressEvent(QMouseEvent *event)
 {
     Q_D(QAbstractSpinBox);
 
+    d->keyboardModifiers = event->modifiers();
     if (event->button() != Qt::LeftButton || d->buttonState != None) {
         return;
     }
 
-    d->updateHoverControl(event->pos());
+    d->updateHoverControl(event->position().toPoint());
     event->accept();
 
     const StepEnabled se = (d->buttonSymbols == NoButtons) ? StepEnabled(StepNone) : stepEnabled();
@@ -1408,6 +1412,7 @@ void QAbstractSpinBox::mouseReleaseEvent(QMouseEvent *event)
 {
     Q_D(QAbstractSpinBox);
 
+    d->keyboardModifiers = event->modifiers();
     if ((d->buttonState & Mouse) != 0)
         d->reset();
     event->accept();
@@ -1421,15 +1426,9 @@ void QAbstractSpinBox::mouseReleaseEvent(QMouseEvent *event)
 */
 
 QAbstractSpinBoxPrivate::QAbstractSpinBoxPrivate()
-    : edit(0), type(QVariant::Invalid), spinClickTimerId(-1),
-      spinClickTimerInterval(100), spinClickThresholdTimerId(-1), spinClickThresholdTimerInterval(-1),
-      effectiveSpinRepeatRate(1), buttonState(None), cachedText(QLatin1String("\x01")),
-      cachedState(QValidator::Invalid), pendingEmit(false), readOnly(false), wrapping(false),
-      ignoreCursorPositionChanged(false), frame(true), accelerate(false), keyboardTracking(true),
-      cleared(false), ignoreUpdateEdit(false), correctionMode(QAbstractSpinBox::CorrectToPreviousValue),
-      stepModifier(Qt::ControlModifier), acceleration(0), hoverControl(QStyle::SC_None),
-      buttonSymbols(QAbstractSpinBox::UpDownArrows), validator(0), showGroupSeparator(0),
-      wheelDeltaRemainder(0)
+    :  pendingEmit(false), readOnly(false), wrapping(false),
+       ignoreCursorPositionChanged(false), frame(true), accelerate(false), keyboardTracking(true),
+       cleared(false), ignoreUpdateEdit(false), showGroupSeparator(false)
 {
 }
 
@@ -1484,7 +1483,7 @@ QStyle::SubControl QAbstractSpinBoxPrivate::newHoverControl(const QPoint &pos)
 
 QString QAbstractSpinBoxPrivate::stripped(const QString &t, int *pos) const
 {
-    QStringRef text(&t);
+    QStringView text(t);
     if (specialValueText.size() == 0 || text != specialValueText) {
         int from = 0;
         int size = text.size();
@@ -1631,6 +1630,8 @@ void QAbstractSpinBoxPrivate::init()
     edit->setValidator(validator);
 
     QStyleOptionSpinBox opt;
+    // ### This is called from the ctor and thus we shouldn't call initStyleOption yet
+    // ### as we only call the base class implementation of initStyleOption called.
     q->initStyleOption(&opt);
     spinClickTimerInterval = q->style()->styleHint(QStyle::SH_SpinBox_ClickAutoRepeatRate, &opt, q);
     spinClickThresholdTimerInterval = q->style()->styleHint(QStyle::SH_SpinBox_ClickAutoRepeatThreshold, &opt, q);
@@ -1678,12 +1679,12 @@ void QAbstractSpinBoxPrivate::updateState(bool up, bool fromKeyboard /* = false
     reset();
     if (q && (q->stepEnabled() & (up ? QAbstractSpinBox::StepUpEnabled
                                   : QAbstractSpinBox::StepDownEnabled))) {
-        spinClickThresholdTimerId = q->startTimer(spinClickThresholdTimerInterval);
         buttonState = (up ? Up : Down) | (fromKeyboard ? Keyboard : Mouse);
         int steps = up ? 1 : -1;
-        if (QGuiApplication::keyboardModifiers() & stepModifier)
+        if (keyboardModifiers & stepModifier)
             steps *= 10;
         q->stepBy(steps);
+        spinClickThresholdTimerId = q->startTimer(spinClickThresholdTimerInterval);
 #ifndef QT_NO_ACCESSIBILITY
         QAccessibleValueChangeEvent event(q, value);
         QAccessible::updateAccessibility(&event);
@@ -1805,7 +1806,7 @@ void QAbstractSpinBoxPrivate::setValue(const QVariant &val, EmitPolicy ep,
 void QAbstractSpinBoxPrivate::updateEdit()
 {
     Q_Q(QAbstractSpinBox);
-    if (type == QVariant::Invalid)
+    if (type == QMetaType::UnknownType)
         return;
     const QString newText = specialValue() ? specialValueText : prefix + textFromValue(value) + suffix;
     if (newText == edit->displayText() || cleared)
@@ -1865,8 +1866,8 @@ QVariant QAbstractSpinBoxPrivate::getZeroVariant() const
 {
     QVariant ret;
     switch (type) {
-    case QVariant::Int: ret = QVariant((int)0); break;
-    case QVariant::Double: ret = QVariant((double)0.0); break;
+    case QMetaType::Int: ret = QVariant(0); break;
+    case QMetaType::Double: ret = QVariant(0.0); break;
     default: break;
     }
     return ret;
@@ -1913,7 +1914,7 @@ QVariant QAbstractSpinBoxPrivate::valueFromText(const QString &) const
 void QAbstractSpinBoxPrivate::interpret(EmitPolicy ep)
 {
     Q_Q(QAbstractSpinBox);
-    if (type == QVariant::Invalid || cleared)
+    if (type == QMetaType::UnknownType || cleared)
         return;
 
     QVariant v = getZeroVariant();
@@ -1954,7 +1955,7 @@ void QAbstractSpinBoxPrivate::clearCache() const
 
 QVariant QAbstractSpinBoxPrivate::calculateAdaptiveDecimalStep(int steps) const
 {
-    Q_UNUSED(steps)
+    Q_UNUSED(steps);
     return singleStep;
 }
 
@@ -2013,11 +2014,11 @@ void QSpinBoxValidator::fixup(QString &input) const
 QVariant operator+(const QVariant &arg1, const QVariant &arg2)
 {
     QVariant ret;
-    if (Q_UNLIKELY(arg1.type() != arg2.type()))
+    if (Q_UNLIKELY(arg1.userType() != arg2.userType()))
         qWarning("QAbstractSpinBox: Internal error: Different types (%s vs %s) (%s:%d)",
                  arg1.typeName(), arg2.typeName(), __FILE__, __LINE__);
-    switch (arg1.type()) {
-    case QVariant::Int: {
+    switch (arg1.userType()) {
+    case QMetaType::Int: {
         const int int1 = arg1.toInt();
         const int int2 = arg2.toInt();
         if (int1 > 0 && (int2 >= INT_MAX - int1)) {
@@ -2031,9 +2032,9 @@ QVariant operator+(const QVariant &arg1, const QVariant &arg2)
         }
         break;
     }
-    case QVariant::Double: ret = QVariant(arg1.toDouble() + arg2.toDouble()); break;
+    case QMetaType::Double: ret = QVariant(arg1.toDouble() + arg2.toDouble()); break;
 #if QT_CONFIG(datetimeparser)
-    case QVariant::DateTime: {
+    case QMetaType::QDateTime: {
         QDateTime a2 = arg2.toDateTime();
         QDateTime a1 = arg1.toDateTime().addDays(QDATETIMEEDIT_DATE_MIN.daysTo(a2.date()));
         a1.setTime(a1.time().addMSecs(a2.time().msecsSinceStartOfDay()));
@@ -2055,13 +2056,13 @@ QVariant operator+(const QVariant &arg1, const QVariant &arg2)
 QVariant operator-(const QVariant &arg1, const QVariant &arg2)
 {
     QVariant ret;
-    if (Q_UNLIKELY(arg1.type() != arg2.type()))
+    if (Q_UNLIKELY(arg1.userType() != arg2.userType()))
         qWarning("QAbstractSpinBox: Internal error: Different types (%s vs %s) (%s:%d)",
                  arg1.typeName(), arg2.typeName(), __FILE__, __LINE__);
-    switch (arg1.type()) {
-    case QVariant::Int: ret = QVariant(arg1.toInt() - arg2.toInt()); break;
-    case QVariant::Double: ret = QVariant(arg1.toDouble() - arg2.toDouble()); break;
-    case QVariant::DateTime: {
+    switch (arg1.userType()) {
+    case QMetaType::Int: ret = QVariant(arg1.toInt() - arg2.toInt()); break;
+    case QMetaType::Double: ret = QVariant(arg1.toDouble() - arg2.toDouble()); break;
+    case QMetaType::QDateTime: {
         QDateTime a1 = arg1.toDateTime();
         QDateTime a2 = arg2.toDateTime();
         int days = a2.daysTo(a1);
@@ -2090,13 +2091,13 @@ QVariant operator*(const QVariant &arg1, double multiplier)
 {
     QVariant ret;
 
-    switch (arg1.type()) {
-    case QVariant::Int:
+    switch (arg1.userType()) {
+    case QMetaType::Int:
         ret = static_cast<int>(qBound<double>(INT_MIN, arg1.toInt() * multiplier, INT_MAX));
         break;
-    case QVariant::Double: ret = QVariant(arg1.toDouble() * multiplier); break;
+    case QMetaType::Double: ret = QVariant(arg1.toDouble() * multiplier); break;
 #if QT_CONFIG(datetimeparser)
-    case QVariant::DateTime: {
+    case QMetaType::QDateTime: {
         double days = QDATETIMEEDIT_DATE_MIN.daysTo(arg1.toDateTime().date()) * multiplier;
         const qint64 daysInt = qint64(days);
         days -= daysInt;
@@ -2119,17 +2120,17 @@ double operator/(const QVariant &arg1, const QVariant &arg2)
     double a1 = 0;
     double a2 = 0;
 
-    switch (arg1.type()) {
-    case QVariant::Int:
+    switch (arg1.userType()) {
+    case QMetaType::Int:
         a1 = (double)arg1.toInt();
         a2 = (double)arg2.toInt();
         break;
-    case QVariant::Double:
+    case QMetaType::Double:
         a1 = arg1.toDouble();
         a2 = arg2.toDouble();
         break;
 #if QT_CONFIG(datetimeparser)
-    case QVariant::DateTime:
+    case QMetaType::QDateTime:
         a1 = QDATETIMEEDIT_DATE_MIN.daysTo(arg1.toDate());
         a2 = QDATETIMEEDIT_DATE_MIN.daysTo(arg2.toDate());
         a1 += arg1.toDateTime().time().msecsSinceStartOfDay() / (36e5 * 24);
@@ -2144,9 +2145,9 @@ double operator/(const QVariant &arg1, const QVariant &arg2)
 
 int QAbstractSpinBoxPrivate::variantCompare(const QVariant &arg1, const QVariant &arg2)
 {
-    switch (arg2.type()) {
-    case QVariant::Date:
-        Q_ASSERT_X(arg1.type() == QVariant::Date, "QAbstractSpinBoxPrivate::variantCompare",
+    switch (arg2.userType()) {
+    case QMetaType::QDate:
+        Q_ASSERT_X(arg1.userType() == QMetaType::QDate, "QAbstractSpinBoxPrivate::variantCompare",
                    qPrintable(QString::fromLatin1("Internal error 1 (%1)").
                               arg(QString::fromLatin1(arg1.typeName()))));
         if (arg1.toDate() == arg2.toDate()) {
@@ -2156,8 +2157,8 @@ int QAbstractSpinBoxPrivate::variantCompare(const QVariant &arg1, const QVariant
         } else {
             return 1;
         }
-    case QVariant::Time:
-        Q_ASSERT_X(arg1.type() == QVariant::Time, "QAbstractSpinBoxPrivate::variantCompare",
+    case QMetaType::QTime:
+        Q_ASSERT_X(arg1.userType() == QMetaType::QTime, "QAbstractSpinBoxPrivate::variantCompare",
                    qPrintable(QString::fromLatin1("Internal error 2 (%1)").
                               arg(QString::fromLatin1(arg1.typeName()))));
         if (arg1.toTime() == arg2.toTime()) {
@@ -2169,7 +2170,7 @@ int QAbstractSpinBoxPrivate::variantCompare(const QVariant &arg1, const QVariant
         }
 
 
-    case QVariant::DateTime:
+    case QMetaType::QDateTime:
         if (arg1.toDateTime() == arg2.toDateTime()) {
             return 0;
         } else if (arg1.toDateTime() < arg2.toDateTime()) {
@@ -2177,7 +2178,7 @@ int QAbstractSpinBoxPrivate::variantCompare(const QVariant &arg1, const QVariant
         } else {
             return 1;
         }
-    case QVariant::Int:
+    case QMetaType::Int:
         if (arg1.toInt() == arg2.toInt()) {
             return 0;
         } else if (arg1.toInt() < arg2.toInt()) {
@@ -2185,7 +2186,7 @@ int QAbstractSpinBoxPrivate::variantCompare(const QVariant &arg1, const QVariant
         } else {
             return 1;
         }
-    case QVariant::Double:
+    case QMetaType::Double:
         if (arg1.toDouble() == arg2.toDouble()) {
             return 0;
         } else if (arg1.toDouble() < arg2.toDouble()) {
@@ -2193,8 +2194,8 @@ int QAbstractSpinBoxPrivate::variantCompare(const QVariant &arg1, const QVariant
         } else {
             return 1;
         }
-    case QVariant::Invalid:
-        if (arg2.type() == QVariant::Invalid)
+    case QMetaType::UnknownType:
+        if (arg2.userType() == QMetaType::UnknownType)
             return 0;
         Q_FALLTHROUGH();
     default:
